@@ -1,6 +1,7 @@
 from __future__ import print_function
 from stretch_body.dynamixel_XL430 import *
 from stretch_body.device import Device
+from stretch_body.trajectories import RevoluteTrajectory
 import time
 from stretch_body.hello_utils import *
 import termios
@@ -58,6 +59,11 @@ class DynamixelHelloXL430(Device):
         self.status={'timestamp_pc':0,'comm_errors':0,'pos':0,'vel':0,'effort':0,'temp':0,'shutdown':0, 'hardware_error':0,
                      'input_voltage_error':0,'overheating_error':0,'motor_encoder_error':0,'electrical_shock_error':0,'overload_error':0,
                      'stalled':0,'stall_overload':0,'pos_ticks':0,'vel_ticks':0,'effort_ticks':0}
+        self.thread_rate_hz = 15.0
+        self.trajectory = RevoluteTrajectory()
+        self._waypoint_ts = None
+        self._waypoint_vel = self.params['motion']['trajectory_max']['vel_r']
+        self._waypoint_accel = self.params['motion']['trajectory_max']['accel_r']
 
         #Share bus resource amongst many XL430s
         self.motor = DynamixelXL430(dxl_id=self.params['id'],
@@ -128,7 +134,8 @@ class DynamixelHelloXL430(Device):
     def do_ping(self, verbose=False):
         return self.motor.do_ping(verbose)
 
-    def startup(self):
+    def startup(self, threaded=False):
+        Device.startup(self, threaded=threaded)
         try:
             if self.motor.do_ping(verbose=False):
                 self.hw_valid = True
@@ -164,8 +171,13 @@ class DynamixelHelloXL430(Device):
             print('DynamixelHelloXL430 Ping failed...', self.name)
             return False
 
+    def _thread_loop(self):
+        self.pull_status()
+        self.update_trajectory()
 
     def stop(self):
+        Device.stop(self)
+        self._waypoint_ts, self._waypoint_vel, self._waypoint_accel = None, None, None
         if self.hw_valid:
             if self.params['disable_torque_on_stop']:
                 self.disable_torque()
@@ -227,8 +239,8 @@ class DynamixelHelloXL430(Device):
                 ts = time.time()
             except(termios.error, DynamixelCommError):
                 #self.logger.warning('Dynamixel communication error on %s: '%self.name)
-                self.port_handler.ser.reset_output_buffer()
-                self.port_handler.ser.reset_input_buffer()
+                self.motor.port_handler.ser.reset_output_buffer()
+                self.motor.port_handler.ser.reset_input_buffer()
                 self.comm_errors.add_error(rx=True,gsr=False)
                 return
         else:
@@ -288,7 +300,20 @@ class DynamixelHelloXL430(Device):
         self.params['zero_t']=x
         self.write_device_params(self.name, self.params)
 
+    def wait_until_at_setpoint(self,timeout=15.0):
+        """Polls for moving status to wait until at commanded position goal
 
+        Returns
+        -------
+        bool
+            True if success, False if timeout
+        """
+        ts = time.time()
+        while time.time() - ts < timeout:
+            if self.motor.get_moving_status() & 1 == 1:
+                return True
+            time.sleep(0.1)
+        return False
 
     def pretty_print(self):
         if not self.hw_valid:
@@ -321,11 +346,11 @@ class DynamixelHelloXL430(Device):
         #self.motor.pretty_print()
 
     def step_sentry(self, robot):
-
         if self.hw_valid and self.robot_params['robot_sentry']['dynamixel_stop_on_runstop'] and self.params['enable_runstop']:
             is_runstopped = robot.pimu.status['runstop_event']
             if is_runstopped is not self.was_runstopped:
                 if is_runstopped:
+                    self.stop_trajectory()
                     self.disable_torque()
                 else:
                     self.enable_torque()
@@ -448,6 +473,94 @@ class DynamixelHelloXL430(Device):
         except (termios.error, DynamixelCommError):
             self.comm_errors.add_error(rx=False, gsr=False)
 
+    # ######### Waypoint Trajectory Interface ##############################
+
+    def follow_trajectory(self, v_r=None, a_r=None, req_calibration=True, move_to_start_point=True):
+        """Starts executing a waypoint trajectory
+
+        `self.trajectory` must be populated with a valid trajectory before calling
+        this method.
+
+        Parameters
+        ----------
+        v_r : float
+            velocity limit for trajectory in radians per second
+        a_r : float
+            acceleration limit for trajectory in radians per second squared
+        req_calibration : bool
+            whether to allow motion prior to homing
+        move_to_start_point : bool
+            whether to move to the trajectory's start to avoid a jump, this
+            time to move doesn't count against the trajectory's timeline
+        """
+        # check if joint valid, homed, and previous trajectory not executing
+        if not self.hw_valid:
+            self.logger.warning('Dynamixel connection to hardware not valid')
+            return False
+        if req_calibration:
+            if not self.is_calibrated:
+                self.logger.warning('Dynamixel not homed')
+                return False
+        if self._waypoint_ts is not None:
+            self.logger.error('Dynamixel waypoint trajectory already active')
+            return False
+
+        # check if trajectory valid
+        vel_limit = v_r if v_r is not None else self.params['motion']['trajectory_max']['vel_r']
+        acc_limit = a_r if a_r is not None else self.params['motion']['trajectory_max']['accel_r']
+        valid, reason = self.trajectory.is_valid(vel_limit, acc_limit)
+        if not valid:
+            self.logger.warning('Dynamixel trajectory not valid: {0}'.format(reason))
+            return False
+        if valid and reason == "must have atleast two waypoints":
+            # skip this device
+            return True
+
+        # set defaults
+        self._waypoint_vel = min(abs(v_r), self.params['motion']['trajectory_max']['vel_r']) \
+            if v_r is not None else self.params['motion']['trajectory_max']['vel_r']
+        self._waypoint_accel = min(abs(a_r), self.params['motion']['trajectory_max']['accel_r']) \
+            if a_r is not None else self.params['motion']['trajectory_max']['accel_r']
+
+        # move to start point
+        if move_to_start_point:
+            self.move_to(self.trajectory[0].position)
+            if not self.wait_until_at_setpoint():
+                self.logger.warning('Dynamixel unable to reach starting point')
+                return False
+
+        # start trajectory
+        self._waypoint_ts = time.time()
+        p0, _, _ = self.trajectory.evaluate_at(time.time() - self._waypoint_ts)
+        self.move_to(p0, self._waypoint_vel, self._waypoint_accel)
+        return True
+
+    def update_trajectory(self):
+        """Updates hardware with the next position goal of `self.trajectory`
+
+        This method must be called frequently to enable complete trajectory execution
+        and preemption of future segments. If used with `stretch_body.robot.Robot` or
+        with `self.startup(threaded=True)`, a background thread is launched for this.
+        Otherwise, the user must handle calling this method.
+        """
+        # check if joint valid, previous trajectory not executing, and not runstopped
+        if not self.hw_valid or self._waypoint_ts is None:
+            return
+        if self.was_runstopped:
+            return
+
+        if (time.time() - self._waypoint_ts) < self.trajectory[-1].time:
+            p1, _, _ = self.trajectory.evaluate_at(time.time() - self._waypoint_ts)
+            self.move_to(p1, self._waypoint_vel, self._waypoint_accel)
+        else:
+            self.move_to(self.trajectory[-1].position, self._waypoint_vel, self._waypoint_accel)
+            self._waypoint_ts, self._waypoint_vel, self._waypoint_accel = None, None, None
+
+    def stop_trajectory(self):
+        """Stop waypoint trajectory immediately and resets hardware
+        """
+        self._waypoint_ts, self._waypoint_vel, self._waypoint_accel = None, None, None
+
 # ##########################################
     """
     Servo calibration works by:
@@ -469,7 +582,7 @@ class DynamixelHelloXL430(Device):
     In single-turn mode, position values are bound to 0-4095 and homing values to -1,024 ~ 1,024
     In addition, the home offset stored during a multi-turn homing procedure may not be valid in single turn (out of range)
     
-    The Home Offset(20) adjusts the home position. The offest value is added to the Present Position(132).
+    The Home Offset(20) adjusts the home position. The offset value is added to the Present Position(132).
     Present Position(132) = Actual Position + Homing Offset(20)
     Range: -1,044,479 ~ 1,044,479
     
