@@ -5,7 +5,7 @@ import importlib
 class EndOfArm(DynamixelXChain):
     """
     The EndOfArm class allows for an extensible serial chain of Dynamixel X series devices
-    It allows the specifc type of device to be declared at runtime via the Yaml parameters
+    It allows the specific type of device to be declared at runtime via the Yaml parameters
     In this way, a user can add their own custom Dynamixel based tools to the robot end-of-arm by
     simply deriving it from DynamixelHelloXL430 and declaring the class name / Python module name
     in the User YAML file
@@ -19,6 +19,8 @@ class EndOfArm(DynamixelXChain):
             dynamixel_device = getattr(importlib.import_module(module_name), class_name)(chain=self)
             self.add_motor(dynamixel_device)
 
+    def startup(self, threaded=True):
+        return DynamixelXChain.startup(self, threaded=threaded)
 
     def get_joint(self, joint_name):
         """Retrieves joint by name.
