@@ -1,6 +1,7 @@
 from __future__ import print_function
 from stretch_body.stepper import Stepper
 from stretch_body.device import Device
+from stretch_body.trajectories import PrismaticTrajectory
 
 import time
 import math
@@ -14,6 +15,8 @@ class Lift(Device):
         Device.__init__(self, 'lift')
         self.motor = Stepper(usb='/dev/hello-motor-lift')
         self.status = {'timestamp_pc':0,'pos': 0.0, 'vel': 0.0, 'force':0.0,'motor': self.motor.status}
+        self.trajectory = PrismaticTrajectory()
+        self.thread_rate_hz = 5.0
 
         # Default controller params
         self.stiffness = 1.0
@@ -28,12 +31,16 @@ class Lift(Device):
         self.motor.set_motion_limits(self.translate_to_motor_rad(self.soft_motion_limits['current'][0]),
                                      self.translate_to_motor_rad(self.soft_motion_limits['current'][1]))
     # ###########  Device Methods #############
-    def startup(self):
-        success= self.motor.startup()
+    def startup(self, threaded=True):
+        Device.startup(self, threaded=threaded)
+        success= self.motor.startup(threaded=False)
         self.__update_status()
         return success
 
     def stop(self):
+        Device.stop(self)
+        if self.motor.hw_valid and int(str(self.motor.board_info['protocol_version'])[1:]) >= 1:
+            self.motor.stop_waypoint_trajectory()
         self.motor.stop()
 
     def pull_status(self):
@@ -49,6 +56,10 @@ class Lift(Device):
     def push_command(self):
         self.motor.push_command()
 
+    def _thread_loop(self):
+        self.pull_status()
+        self.update_trajectory()
+
     def pretty_print(self):
         print('----- Lift ------ ')
         print('Pos (m): ', self.status['pos'])
@@ -126,15 +137,13 @@ class Lift(Device):
         else:
             a_r = self.accel_r
 
-        if contact_thresh_neg_N is not None:
-            i_contact_neg = max(self.translate_force_to_motor_current(contact_thresh_neg_N),self.params['contact_thresh_max_N'][0])
-        else:
-            i_contact_neg = self.i_contact_neg
+        i_contact_neg = self.translate_force_to_motor_current(
+            max(contact_thresh_neg_N, self.params['contact_thresh_max_N'][0])) \
+            if contact_thresh_neg_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][0])
+        i_contact_pos = self.translate_force_to_motor_current(
+            min(contact_thresh_pos_N, self.params['contact_thresh_max_N'][1])) \
+            if contact_thresh_pos_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][1])
 
-        if contact_thresh_pos_N is not None:
-            i_contact_pos = min(self.translate_force_to_motor_current(contact_thresh_pos_N),self.params['contact_thresh_max_N'][1])
-        else:
-            i_contact_pos = self.i_contact_pos
 
         self.motor.set_command(mode=Stepper.MODE_VEL_TRAJ,
                                v_des=v_r,
@@ -174,17 +183,12 @@ class Lift(Device):
         else:
             a_r = self.accel_r
 
-        if contact_thresh_neg_N is not None:
-            i_contact_neg = max(self.translate_force_to_motor_current(contact_thresh_neg_N),
-                                self.params['contact_thresh_max_N'][0])
-        else:
-            i_contact_neg = self.i_contact_neg
-
-        if contact_thresh_pos_N is not None:
-            i_contact_pos = min(self.translate_force_to_motor_current(contact_thresh_pos_N),
-                                self.params['contact_thresh_max_N'][1])
-        else:
-            i_contact_pos = self.i_contact_pos
+        i_contact_neg = self.translate_force_to_motor_current(
+            max(contact_thresh_neg_N, self.params['contact_thresh_max_N'][0])) \
+            if contact_thresh_neg_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][0])
+        i_contact_pos = self.translate_force_to_motor_current(
+            min(contact_thresh_pos_N, self.params['contact_thresh_max_N'][1])) \
+            if contact_thresh_pos_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][1])
 
         self.motor.set_command(mode = Stepper.MODE_POS_TRAJ,
                                 x_des=self.translate_to_motor_rad(x_m),
@@ -230,17 +234,12 @@ class Lift(Device):
         else:
             a_r = self.accel_r
 
-        if contact_thresh_neg_N is not None:
-            i_contact_neg = max(self.translate_force_to_motor_current(contact_thresh_neg_N),
-                                self.params['contact_thresh_max_N'][0])
-        else:
-            i_contact_neg = self.i_contact_neg
-
-        if contact_thresh_pos_N is not None:
-            i_contact_pos = min(self.translate_force_to_motor_current(contact_thresh_pos_N),
-                                self.params['contact_thresh_max_N'][1])
-        else:
-            i_contact_pos = self.i_contact_pos
+        i_contact_neg = self.translate_force_to_motor_current(
+            max(contact_thresh_neg_N, self.params['contact_thresh_max_N'][0])) \
+            if contact_thresh_neg_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][0])
+        i_contact_pos = self.translate_force_to_motor_current(
+            min(contact_thresh_pos_N, self.params['contact_thresh_max_N'][1])) \
+            if contact_thresh_pos_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][1])
 
         #print('Lift %.2f , %.2f  , %.2f' % (x_m, self.motor_rad_to_translate_m(v_r), self.motor_rad_to_translate_m(a_r)))
 
@@ -253,6 +252,117 @@ class Lift(Device):
                                 i_contact_pos=i_contact_pos,
                                 i_contact_neg=i_contact_neg)
 
+    # ######### Waypoint Trajectory Interface ##############################
+
+    def follow_trajectory(self, v_m=None, a_m=None, stiffness=None, contact_thresh_pos_N=None, contact_thresh_neg_N=None,
+                          req_calibration=True, move_to_start_point=True):
+        """Starts executing a waypoint trajectory
+
+        `self.trajectory` must be populated with a valid trajectory before calling
+        this method.
+
+        Parameters
+        ----------
+        v_m : float
+            velocity limit for trajectory in meters per second
+        a_m : float
+            acceleration limit for trajectory in meters per second squared
+        stiffness : float
+            stiffness of motion. Range 0.0 (min) to 1.0 (max)
+        contact_thresh_pos_N : float
+            force threshold to stop motion (~Newtons, up direction)
+        contact_thresh_neg_N : float
+            force threshold to stop motion (~Newtons, down direction)
+        req_calibration : bool
+            whether to allow motion prior to homing
+        move_to_start_point : bool
+            whether to move to the trajectory's start to avoid a jump, this
+            time to move doesn't count against the trajectory's timeline
+        """
+        # check if joint valid, homed, and right protocol
+        if not self.motor.hw_valid:
+            self.logger.warning('Lift connection to hardware not valid')
+            return False
+        if req_calibration:
+            if not self.motor.status['pos_calibrated']:
+                self.logger.warning('Lift not calibrated')
+                return False
+        if int(str(self.motor.board_info['protocol_version'])[1:]) < 1:
+            self.logger.warning("Lift firmware version doesn't support waypoint trajectories")
+            return False
+
+        # check if trajectory valid
+        vel_limit = v_m if v_m is not None else self.params['motion']['trajectory_max']['vel_m']
+        acc_limit = a_m if a_m is not None else self.params['motion']['trajectory_max']['accel_m']
+        valid, reason = self.trajectory.is_valid(vel_limit, acc_limit)
+        if not valid:
+            self.logger.warning('Lift traj not valid: {0}'.format(reason))
+            return False
+        if valid and reason == "must have atleast two waypoints":
+            # skip this device
+            return True
+
+        # set defaults
+        stiffness = max(0, min(1.0, stiffness)) if stiffness is not None else self.stiffness
+        v_r = self.translate_to_motor_rad(min(abs(v_m), self.params['motion']['trajectory_max']['vel_m'])) \
+            if v_m is not None else self.translate_to_motor_rad(self.params['motion']['trajectory_max']['vel_m'])
+        a_r = self.translate_to_motor_rad(min(abs(a_m), self.params['motion']['trajectory_max']['accel_m'])) \
+            if a_m is not None else self.translate_to_motor_rad(self.params['motion']['trajectory_max']['accel_m'])
+        i_contact_neg = self.translate_force_to_motor_current(max(contact_thresh_neg_N, self.params['contact_thresh_max_N'][0])) \
+            if contact_thresh_neg_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][0])
+        i_contact_pos = self.translate_force_to_motor_current(min(contact_thresh_pos_N, self.params['contact_thresh_max_N'][1])) \
+            if contact_thresh_pos_N is not None else self.translate_force_to_motor_current(self.params['contact_thresh_N'][1])
+
+        # move to start point
+        if move_to_start_point:
+            prev_sync_mode = self.motor.gains['enable_sync_mode']
+            self.move_to(self.trajectory[0].position)
+            self.motor.disable_sync_mode()
+            self.push_command()
+            if not self.motor.wait_until_at_setpoint():
+                self.logger.warning('Lift unable to reach starting point')
+            if prev_sync_mode:
+                self.motor.enable_sync_mode()
+                self.push_command()
+
+        # start trajectory
+        self.motor.set_command(mode=Stepper.MODE_POS_TRAJ_WAYPOINT,
+                               v_des=v_r,
+                               a_des=a_r,
+                               stiffness=stiffness,
+                               i_contact_pos=i_contact_pos,
+                               i_contact_neg=i_contact_neg)
+        self.motor.push_command()
+        s0 = self.trajectory.get_segment(0, to_motor_rad=self.translate_to_motor_rad).to_array()
+        return self.motor.start_waypoint_trajectory(s0)
+
+    def update_trajectory(self):
+        """Updates hardware with the next segment of `self.trajectory`
+
+        This method must be called frequently to enable complete trajectory execution
+        and preemption of future segments. If used with `stretch_body.robot.Robot` or
+        with `self.startup(threaded=True)`, a background thread is launched for this.
+        Otherwise, the user must handle calling this method.
+        """
+        # check if joint valid, right protocol, and right mode
+        if not self.motor.hw_valid or int(str(self.motor.board_info['protocol_version'])[1:]) < 1:
+            return
+        if self.motor.status['mode'] != self.motor.MODE_POS_TRAJ_WAYPOINT:
+            return
+
+        if self.motor.status['waypoint_traj']['state'] == 'active':
+            next_segment_id = self.motor.status['waypoint_traj']['segment_id'] - 2 + 1 # subtract 2 due to IDs 0 & 1 being reserved by firmware
+            if next_segment_id < self.trajectory.get_num_segments():
+                s1 = self.trajectory.get_segment(next_segment_id, to_motor_rad=self.translate_to_motor_rad).to_array()
+                self.motor.set_next_trajectory_segment(s1)
+        elif self.motor.status['waypoint_traj']['state'] == 'idle' and self.motor.status['mode'] == Stepper.MODE_POS_TRAJ_WAYPOINT:
+            self.motor.enable_pos_traj()
+            self.push_command()
+
+    def stop_trajectory(self):
+        """Stop waypoint trajectory immediately and resets hardware
+        """
+        self.motor.stop_waypoint_trajectory()
 
     # ######### Utility ##############################
 
@@ -293,6 +403,8 @@ class Lift(Device):
             self.logger.warning('Not able to home lift. Hardware not present')
             return
         print('Homing lift...')
+        prev_guarded_mode = self.motor.gains['enable_guarded_mode']
+        prev_sync_mode = self.motor.gains['enable_sync_mode']
         self.motor.enable_guarded_mode()
         self.motor.disable_sync_mode()
         self.motor.reset_pos_calibrated()
@@ -326,10 +438,10 @@ class Lift(Device):
         self.push_command()
         time.sleep(6.0)
 
-        #Restore default
-        if not self.motor.gains['enable_guarded_mode']:
+        # Restore previous modes
+        if not prev_guarded_mode:
             self.motor.disable_guarded_mode()
-        if self.motor.gains['enable_sync_mode']:
+        if prev_sync_mode:
             self.motor.enable_sync_mode()
         self.push_command()
 
