@@ -5,8 +5,9 @@ from stretch_body.hello_utils import *
 import textwrap
 import threading
 import sys
+import time
 
-# ######################## WACC #################################
+# ######################## STEPPER #################################
 
 class StepperBase(Device):
     """
@@ -32,6 +33,12 @@ class StepperBase(Device):
     RPC_REPLY_STEPPER_BOARD_INFO = 18
     RPC_SET_MOTION_LIMITS = 19
     RPC_REPLY_MOTION_LIMITS = 20
+    RPC_SET_NEXT_TRAJECTORY_SEG =21
+    RPC_REPLY_SET_NEXT_TRAJECTORY_SEG =22
+    RPC_START_NEW_TRAJECTORY =23
+    RPC_REPLY_START_NEW_TRAJECTORY =24
+    RPC_RESET_TRAJECTORY =25
+    RPC_REPLY_RESET_TRAJECTORY =26
 
     MODE_SAFETY = 0
     MODE_FREEWHEEL = 1
@@ -42,19 +49,36 @@ class StepperBase(Device):
     MODE_VEL_TRAJ = 6
     MODE_CURRENT = 7
     MODE_POS_TRAJ_INCR = 8
+    MODE_POS_TRAJ_WAYPOINT=9
 
-    DIAG_POS_CALIBRATED = 1  # Has a pos zero RPC been recieved since powerup
+    MODE_NAMES = {
+        MODE_SAFETY: 'MODE_SAFETY',
+        MODE_FREEWHEEL: 'MODE_FREEWHEEL',
+        MODE_HOLD: 'MODE_HOLD',
+        MODE_POS_PID: 'MODE_POS_PID',
+        MODE_VEL_PID: 'MODE_VEL_PID',
+        MODE_POS_TRAJ: 'MODE_POS_TRAJ',
+        MODE_VEL_TRAJ: 'MODE_VEL_TRAJ',
+        MODE_CURRENT: 'MODE_CURRENT',
+        MODE_POS_TRAJ_INCR: 'MODE_POS_TRAJ_INCR',
+        MODE_POS_TRAJ_WAYPOINT: 'MODE_POS_TRAJ_WAYPOINT',
+    }
+
+    DIAG_POS_CALIBRATED = 1  # Has a pos zero RPC been received since powerup
     DIAG_RUNSTOP_ON = 2  # Is controller in runstop mode
     DIAG_NEAR_POS_SETPOINT = 4  # Is pos controller within gains.pAs_d of setpoint
     DIAG_NEAR_VEL_SETPOINT = 8  # Is vel controller within gains.vAs_d of setpoint
     DIAG_IS_MOVING = 16  # Is measured velocity greater than gains.vAs_d
     DIAG_AT_CURRENT_LIMIT = 32  # Is controller current saturated
-    DIAG_IS_MG_ACCELERATING = 64  # Is controler motion generator acceleration non-zero
+    DIAG_IS_MG_ACCELERATING = 64  # Is controller motion generator acceleration non-zero
     DIAG_IS_MG_MOVING = 128  # Is controller motion generator velocity non-zero
     DIAG_CALIBRATION_RCVD = 256  # Is calibration table in flash
-    DIAG_IN_GUARDED_EVENT = 512  # Guarded event occured during motion
+    DIAG_IN_GUARDED_EVENT = 512  # Guarded event occurred during motion
     DIAG_IN_SAFETY_EVENT = 1024  # Is it forced into safety mode
     DIAG_WAITING_ON_SYNC = 2048  # Command received but no sync yet
+    DIAG_TRAJ_ACTIVE     = 4096     # Whether a waypoint trajectory is actively executing
+    DIAG_TRAJ_WAITING_ON_SYNC = 8192 # Currently waiting on a sync signal before starting trajectory
+    DIAG_IN_SYNC_MODE = 16384        # Currently running in sync mode
 
     CONFIG_SAFETY_HOLD = 1  # Hold position in safety mode? Otherwise freewheel
     CONFIG_ENABLE_RUNSTOP = 2  # Recognize runstop signal?
@@ -79,13 +103,18 @@ class StepperBase(Device):
         self.status = {'mode': 0, 'effort': 0, 'current':0,'pos': 0, 'vel': 0, 'err':0,'diag': 0,'timestamp': 0, 'debug':0,'guarded_event':0,
                        'transport': self.transport.status,'pos_calibrated':0,'runstop_on':0,'near_pos_setpoint':0,'near_vel_setpoint':0,
                        'is_moving':0,'is_moving_filtered':0,'at_current_limit':0,'is_mg_accelerating':0,'is_mg_moving':0,'calibration_rcvd': 0,'in_guarded_event':0,
-                       'in_safety_event':0,'waiting_on_sync':0}
+                       'in_safety_event':0,'waiting_on_sync':0,
+                       'waypoint_traj':{'state':'idle','setpoint':None, 'segment_id':0,}}
         self.board_info={'board_version':None, 'firmware_version':None,'protocol_version':None}
-        self.mode_names={self.MODE_SAFETY:'MODE_SAFETY', self.MODE_FREEWHEEL:'MODE_FREEWHEEL',self.MODE_HOLD:'MODE_HOLD',self.MODE_POS_PID:'MODE_POS_PID',
-                         self.MODE_VEL_PID:'MODE_VEL_PID',self.MODE_POS_TRAJ:'MODE_POS_TRAJ',self.MODE_VEL_TRAJ:'MODE_VEL_TRAJ',self.MODE_CURRENT:'MODE_CURRENT', self.MODE_POS_TRAJ_INCR:'MODE_POS_TRAJ_INCR'}
         self.motion_limits=[0,0]
         self.is_moving_history = [False] * 10
 
+        self._waypoint_traj_segment = [0] * 8
+        self._waypoint_traj_start_success = False
+        self._waypoint_traj_set_next_traj_success = False
+        self._waypoint_traj_start_error_msg = ""
+        self._waypoint_traj_set_next_error_msg = ""
+
         self._dirty_command = False
         self._dirty_gains = False
         self._dirty_trigger = False
@@ -98,7 +127,8 @@ class StepperBase(Device):
         self.gains = self.params['gains'].copy()
 
     # ###########  Device Methods #############
-    def startup(self):
+    def startup(self, threaded=False):
+        Device.startup(self, threaded=threaded)
         with self.lock:
             self.hw_valid = self.transport.startup()
             if self.hw_valid:
@@ -111,6 +141,7 @@ class StepperBase(Device):
 
     #Configure control mode prior to calling this on process shutdown (or default to freewheel)
     def stop(self):
+        Device.stop(self)
         if not self.hw_valid:
             return
         with self.lock:
@@ -167,38 +198,8 @@ class StepperBase(Device):
             self.transport.step(exiting=exiting)
 
     def pretty_print(self):
-        print('-----------')
-        print('Mode',self.mode_names[self.status['mode']])
-        print('x_des (rad)', self._command['x_des'], '(deg)',rad_to_deg(self._command['x_des']))
-        print('v_des (rad)', self._command['v_des'], '(deg)',rad_to_deg(self._command['v_des']))
-        print('a_des (rad)', self._command['a_des'], '(deg)',rad_to_deg(self._command['a_des']))
-        print('Stiffness',self._command['stiffness'])
-        print('Feedforward', self._command['i_feedforward'])
-        print('Pos (rad)', self.status['pos'], '(deg)',rad_to_deg(self.status['pos']))
-        print('Vel (rad/s)', self.status['vel'], '(deg)',rad_to_deg(self.status['vel']))
-        print('Effort', self.status['effort'])
-        print('Current (A)', self.status['current'])
-        print('Error (deg)', rad_to_deg(self.status['err']))
-        print('Debug', self.status['debug'])
-        print('Guarded Events:', self.status['guarded_event'])
-        print('Diag', self.status['diag'])
-        print('       Position Calibrated:', self.status['pos_calibrated'])
-        print('       Runstop on:', self.status['runstop_on'])
-        print('       Near Pos Setpoint:', self.status['near_pos_setpoint'])
-        print('       Near Vel Setpoint:', self.status['near_vel_setpoint'])
-        print('       Is Moving:', self.status['is_moving'])
-        print('       Is Moving Filtered:', self.status['is_moving_filtered'])
-        print('       At Current Limit:', self.status['at_current_limit'])
-        print('       Is MG Accelerating:', self.status['is_mg_accelerating'])
-        print('       Is MG Moving:', self.status['is_mg_moving'])
-        print('       Encoder Calibration in Flash:', self.status['calibration_rcvd'])
-        print('       In Guarded Event:', self.status['in_guarded_event'])
-        print('       In Safety Event:', self.status['in_safety_event'])
-        print('       Waiting on Sync:', self.status['waiting_on_sync'])
-        print('Timestamp (s)', self.status['timestamp'])
-        print('Read error', self.transport.status['read_error'])
-        print('Board version:', self.board_info['board_version'])
-        print('Firmware version:', self.board_info['firmware_version'])
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
 
     def step_sentry(self, robot):
         if self.hw_valid and self.robot_params['robot_sentry']['stepper_is_moving_filter']:
@@ -290,6 +291,10 @@ class StepperBase(Device):
     def enable_pos_traj(self):
         self.set_command(mode=self.MODE_POS_TRAJ, x_des=self.status['pos'])
 
+    def enable_pos_traj_waypoint(self):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
+
     def enable_pos_traj_incr(self):
         self.set_command(mode=self.MODE_POS_TRAJ_INCR, x_des=0)
 
@@ -374,11 +379,17 @@ class StepperBase(Device):
 
 
     def wait_until_at_setpoint(self,timeout=15.0):
+        """
+        Poll until near setpoint
+        Return True if success
+        Return False if timeout
+        """
         ts = time.time()
         self.pull_status()
         while not self.status['near_pos_setpoint'] and time.time() - ts < timeout:
             time.sleep(0.1)
             self.pull_status()
+        return self.status['near_pos_setpoint']
 
     def current_to_effort(self,i_A):
         mA_per_tick = (3300 / 255) / (10 * 0.1)
@@ -474,7 +485,7 @@ class StepperBase(Device):
         if reply[0] != self.RPC_REPLY_ENC_CALIB:
             self.logger.debug('Error RPC_REPLY_ENC_CALIB', reply[0])
 
-    # ######################Menu Inteface ################################3
+    # ######################Menu Interface ################################3
 
 
     def turn_rpc_interface_on(self):
@@ -509,6 +520,18 @@ class StepperBase(Device):
                 reply.append(r)
             return reply
 
+    # ################Waypoint Trajectory Interface #####################
+    def start_waypoint_trajectory(self, first_segment):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
+
+    def set_next_trajectory_segment(self, next_segment):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
+
+    def stop_waypoint_trajectory(self):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
 
     # ################Transport Callbacks #####################
     def unpack_board_info(self,s):
@@ -521,6 +544,10 @@ class StepperBase(Device):
             sidx += 20
             return sidx
 
+    def unpack_status(self,s):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
+
     def unpack_gains(self,s):
         with self.lock:
             sidx=0
@@ -630,7 +657,7 @@ class StepperBase(Device):
             sidx += 4
             return sidx
 
-    def unpack_status(self,s):
+    def pack_trajectory_segment(self, s, sidx):
         raise NotImplementedError('This method not supported for firmware on protocol {0}.'
             .format(self.board_info['protocol_version']))
 
@@ -682,6 +709,18 @@ class StepperBase(Device):
         else:
             print('Error RPC_REPLY_READ_GAINS_FROM_FLASH', reply[0])
 
+    def rpc_start_new_traj_reply(self, reply):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
+
+    def rpc_set_next_traj_seg_reply(self, reply):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
+
+    def rpc_reset_traj_reply(self, reply):
+        raise NotImplementedError('This method not supported for firmware on protocol {0}.'
+            .format(self.board_info['protocol_version']))
+
 # ######################## STEPPER PROTOCOL PO #################################
 
 class Stepper_Protocol_P0(StepperBase):
@@ -713,6 +752,40 @@ class Stepper_Protocol_P0(StepperBase):
             self.status['waiting_on_sync'] = self.status['diag'] & self.DIAG_WAITING_ON_SYNC > 0
             return sidx
 
+    def pretty_print(self):
+        print('-----------')
+        print('Mode', self.MODE_NAMES[self.status['mode']])
+        print('x_des (rad)', self._command['x_des'], '(deg)',rad_to_deg(self._command['x_des']))
+        print('v_des (rad)', self._command['v_des'], '(deg)',rad_to_deg(self._command['v_des']))
+        print('a_des (rad)', self._command['a_des'], '(deg)',rad_to_deg(self._command['a_des']))
+        print('Stiffness',self._command['stiffness'])
+        print('Feedforward', self._command['i_feedforward'])
+        print('Pos (rad)', self.status['pos'], '(deg)',rad_to_deg(self.status['pos']))
+        print('Vel (rad/s)', self.status['vel'], '(deg)',rad_to_deg(self.status['vel']))
+        print('Effort', self.status['effort'])
+        print('Current (A)', self.status['current'])
+        print('Error (deg)', rad_to_deg(self.status['err']))
+        print('Debug', self.status['debug'])
+        print('Guarded Events:', self.status['guarded_event'])
+        print('Diag', format(self.status['diag'], '032b'))
+        print('       Position Calibrated:', self.status['pos_calibrated'])
+        print('       Runstop on:', self.status['runstop_on'])
+        print('       Near Pos Setpoint:', self.status['near_pos_setpoint'])
+        print('       Near Vel Setpoint:', self.status['near_vel_setpoint'])
+        print('       Is Moving:', self.status['is_moving'])
+        print('       Is Moving Filtered:', self.status['is_moving_filtered'])
+        print('       At Current Limit:', self.status['at_current_limit'])
+        print('       Is MG Accelerating:', self.status['is_mg_accelerating'])
+        print('       Is MG Moving:', self.status['is_mg_moving'])
+        print('       Encoder Calibration in Flash:', self.status['calibration_rcvd'])
+        print('       In Guarded Event:', self.status['in_guarded_event'])
+        print('       In Safety Event:', self.status['in_safety_event'])
+        print('       Waiting on Sync:', self.status['waiting_on_sync'])
+        print('Timestamp (s)', self.status['timestamp'])
+        print('Read error', self.transport.status['read_error'])
+        print('Board version:', self.board_info['board_version'])
+        print('Firmware version:', self.board_info['firmware_version'])
+
 # ######################## STEPPER PROTOCOL P1 #################################
 class Stepper_Protocol_P1(StepperBase):
     def unpack_status(self,s):
@@ -728,8 +801,8 @@ class Stepper_Protocol_P1(StepperBase):
             self.status['timestamp'] = self.timestamp.set(unpack_uint64_t(s[sidx:]));sidx += 8
             self.status['debug'] = unpack_float_t(s[sidx:]);sidx += 4
             self.status['guarded_event'] = unpack_uint32_t(s[sidx:]);sidx += 4
-            traj_setpoint = unpack_float_t(s[sidx:]);sidx += 4
-            traj_id = unpack_uint16_t(s[sidx:]);sidx += 2
+            self.status['waypoint_traj']['setpoint'] = unpack_float_t(s[sidx:]);sidx += 4
+            self.status['waypoint_traj']['segment_id'] = unpack_uint16_t(s[sidx:]);sidx += 2
 
             self.status['pos_calibrated'] =self.status['diag'] & self.DIAG_POS_CALIBRATED > 0
             self.status['runstop_on'] =self.status['diag'] & self.DIAG_RUNSTOP_ON > 0
@@ -743,11 +816,162 @@ class Stepper_Protocol_P1(StepperBase):
             self.status['in_guarded_event'] = self.status['diag'] & self.DIAG_IN_GUARDED_EVENT > 0
             self.status['in_safety_event'] = self.status['diag'] & self.DIAG_IN_SAFETY_EVENT > 0
             self.status['waiting_on_sync'] = self.status['diag'] & self.DIAG_WAITING_ON_SYNC > 0
+            self.status['in_sync_mode'] = self.status['diag'] & self.DIAG_IN_SYNC_MODE > 0
+
+            if self.status['diag'] & self.DIAG_TRAJ_WAITING_ON_SYNC > 0:
+                self.status['waypoint_traj']['state']='waiting_on_sync'
+            elif self.status['diag'] & self.DIAG_TRAJ_ACTIVE > 0:
+                self.status['waypoint_traj']['state']='active'
+            else:
+                self.status['waypoint_traj']['state']='idle'
             return sidx
 
+    def pretty_print(self):
+        print('-----------')
+        print('Mode', self.MODE_NAMES[self.status['mode']])
+        print('x_des (rad)', self._command['x_des'], '(deg)',rad_to_deg(self._command['x_des']))
+        print('v_des (rad)', self._command['v_des'], '(deg)',rad_to_deg(self._command['v_des']))
+        print('a_des (rad)', self._command['a_des'], '(deg)',rad_to_deg(self._command['a_des']))
+        print('Stiffness',self._command['stiffness'])
+        print('Feedforward', self._command['i_feedforward'])
+        print('Pos (rad)', self.status['pos'], '(deg)',rad_to_deg(self.status['pos']))
+        print('Vel (rad/s)', self.status['vel'], '(deg)',rad_to_deg(self.status['vel']))
+        print('Effort', self.status['effort'])
+        print('Current (A)', self.status['current'])
+        print('Error (deg)', rad_to_deg(self.status['err']))
+        print('Debug', self.status['debug'])
+        print('Guarded Events:', self.status['guarded_event'])
+        print('Diag', format(self.status['diag'], '032b'))
+        print('       Position Calibrated:', self.status['pos_calibrated'])
+        print('       Runstop on:', self.status['runstop_on'])
+        print('       Near Pos Setpoint:', self.status['near_pos_setpoint'])
+        print('       Near Vel Setpoint:', self.status['near_vel_setpoint'])
+        print('       Is Moving:', self.status['is_moving'])
+        print('       Is Moving Filtered:', self.status['is_moving_filtered'])
+        print('       At Current Limit:', self.status['at_current_limit'])
+        print('       Is MG Accelerating:', self.status['is_mg_accelerating'])
+        print('       Is MG Moving:', self.status['is_mg_moving'])
+        print('       Encoder Calibration in Flash:', self.status['calibration_rcvd'])
+        print('       In Guarded Event:', self.status['in_guarded_event'])
+        print('       In Safety Event:', self.status['in_safety_event'])
+        print('       Waiting on Sync:', self.status['waiting_on_sync'])
+        print('Waypoint Trajectory')
+        print('       State:', self.status['waypoint_traj']['state'])
+        print('       Setpoint: (rad) %s | (deg) %s'%(self.status['waypoint_traj']['setpoint'],  rad_to_deg(self.status['waypoint_traj']['setpoint'])))
+        print('       Segment ID:', self.status['waypoint_traj']['segment_id'])
+        print('Timestamp (s)', self.status['timestamp'])
+        print('Read error', self.transport.status['read_error'])
+        print('Board version:', self.board_info['board_version'])
+        print('Firmware version:', self.board_info['firmware_version'])
+
+    def enable_pos_traj_waypoint(self):
+        self.set_command(mode=self.MODE_POS_TRAJ_WAYPOINT)
+
+    def start_waypoint_trajectory(self, first_segment):
+        """Starts execution of a waypoint trajectory on hardware
+
+        Parameters
+        ----------
+        first_segment : list
+            List of length eight, structured like [duration_s, a0, a1, a2, a3, a4, a5, segment_id].
+            The hardware begins executing this first segment of a spline. The segment's duration and
+            six coefficients (a0-a5) fill the first seven elements of the list. A segment ID, always
+            2 for the first segment, fills the last element of the list.
+
+        Returns
+        -------
+        bool
+            True if uC successfully initiated a new trajectory
+        """
+        if len(first_segment) != 8:
+            self.logger.warning('start_waypoint_trajectory: Invalid segment arr length (must be 8)')
+            return False
+        self._waypoint_traj_segment = first_segment
+        with self.lock:
+            if self._waypoint_traj_segment is not None:
+                self.transport.payload_out[0] = self.RPC_START_NEW_TRAJECTORY
+                sidx = self.pack_trajectory_segment(self.transport.payload_out, 1)
+                self.transport.queue_rpc2(sidx, self.rpc_start_new_traj_reply)
+            self.transport.step2()
+            if not self._waypoint_traj_start_success:
+                self.logger.warning('start_waypoint_trajectory: %s' % self._waypoint_traj_start_error_msg.capitalize())
+            return self._waypoint_traj_start_success
+
+    def set_next_trajectory_segment(self, next_segment):
+        """Sets the next segment for the hardware to execute
+
+        This method is generally called multiple times while the prior segment is executing. This
+        provides the hardware with the next segment to gracefully transition across the entire spline,
+        while allowing users to preempt or modify the future trajectory in real time.
+
+        This method will return False if there is not already an segment executing on the uC
+
+        Parameters
+        ----------
+        next_segment : list
+            List of length eight, structured like [duration_s, a0, a1, a2, a3, a4, a5, segment_id].
+            Duration and six coefficients fill the first seven elements of the list. Generally, the
+            coefficients are calculated to smoothly transition across a spline. The segment ID, always
+            1 higher than the prior segment's ID, fills the last element of the list.
+
+        Returns
+        -------
+        bool
+            True if uC successfully queued next trajectory
+        """
+        if len(next_segment) != 8:
+            self.logger.warning('set_next_trajectory_segment: Invalid segment arr length (must be 8)')
+            return False
+        self._waypoint_traj_segment = next_segment
+        with self.lock:
+            if self._waypoint_traj_segment is not None:
+                self.transport.payload_out[0] = self.RPC_SET_NEXT_TRAJECTORY_SEG
+                sidx = self.pack_trajectory_segment(self.transport.payload_out, 1)
+                self.transport.queue_rpc2(sidx, self.rpc_set_next_traj_seg_reply)
+            self.transport.step2()
+            if not self._waypoint_traj_set_next_traj_success:
+                self.logger.warning('set_next_trajectory_segment: %s' % self._waypoint_traj_set_next_error_msg.capitalize())
+            return self._waypoint_traj_set_next_traj_success
+
+    def stop_waypoint_trajectory(self):
+        """Stops execution of the waypoint trajectory running in hardware
+        """
+        with self.lock:
+            self.transport.payload_out[0] = self.RPC_RESET_TRAJECTORY
+            self.transport.queue_rpc2(1, self.rpc_reset_traj_reply)
+            self.transport.step2()
+
+    def pack_trajectory_segment(self, s, sidx):
+        with self.lock:
+            for i in range(7):
+                pack_float_t(s, sidx, self._waypoint_traj_segment[i]); sidx += 4
+            pack_uint8_t(s, sidx, self._waypoint_traj_segment[7]); sidx += 1
+            return sidx
+
+    def rpc_start_new_traj_reply(self, reply):
+        if reply[0] == self.RPC_REPLY_START_NEW_TRAJECTORY:
+            with self.lock:
+                sidx=1
+                self._waypoint_traj_start_success = unpack_uint8_t(reply[sidx:]); sidx += 1
+                self._waypoint_traj_start_error_msg = unpack_string_t(reply[sidx:], 100).strip('\x00'); sidx += 100
+        else:
+            self.logger.error('RPC_REPLY_START_NEW_TRAJECTORY replied {0}'.format(reply[0]))
+
+    def rpc_set_next_traj_seg_reply(self, reply):
+        if reply[0] == self.RPC_REPLY_SET_NEXT_TRAJECTORY_SEG:
+            with self.lock:
+                sidx=1
+                self._waypoint_traj_set_next_traj_success = unpack_uint8_t(reply[sidx:]); sidx += 1
+                self._waypoint_traj_set_next_error_msg = unpack_string_t(reply[sidx:], 100).strip('\x00'); sidx += 100
+        else:
+            self.logger.error('RPC_REPLY_SET_NEXT_TRAJECTORY_SEG replied {0}'.format(reply[0]))
+
+    def rpc_reset_traj_reply(self, reply):
+        if reply[0] != self.RPC_REPLY_RESET_TRAJECTORY:
+            self.logger.error('RPC_REPLY_RESET_TRAJECTORY replied {0}'.format(reply[0]))
 
 
-# ######################## PIMU #################################
+# ######################## STEPPER #################################
 class Stepper(StepperBase):
     """
     API to the Stretch RE1 Power and IMU board (Pimu)
@@ -757,12 +981,12 @@ class Stepper(StepperBase):
         # Order in descending order so more recent protocols/methods override less recent
         self.supported_protocols = {'p0': (Stepper_Protocol_P0,), 'p1': (Stepper_Protocol_P1,Stepper_Protocol_P0,)}
 
-    def startup(self):
+    def startup(self, threaded=False):
         """
         First determine which protocol version the uC firmware is running.
         Based on that version, replaces PimuBase class inheritance with a inheritance to a child class of PimuBase that supports that protocol
         """
-        StepperBase.startup(self)
+        StepperBase.startup(self, threaded=threaded)
         if self.hw_valid:
             if self.board_info['protocol_version'] in self.supported_protocols:
                 Stepper.__bases__ = self.supported_protocols[self.board_info['protocol_version']]
