@@ -49,7 +49,9 @@ class WaccBase(Device):
         self.hw_valid = False
 
     # ###########  Device Methods #############
-    def startup(self):
+
+    def startup(self, threaded=False):
+        Device.startup(self, threaded=threaded)
         with self.lock:
             self.hw_valid = self.transport.startup()
             if self.hw_valid:
@@ -61,6 +63,7 @@ class WaccBase(Device):
             return False
 
     def stop(self):
+        Device.stop(self)
         if not self.hw_valid:
             return
         with self.lock:
@@ -257,12 +260,12 @@ class Wacc(WaccBase):
         #Order in descending order so more recent protocols/methods override less recent
         self.supported_protocols = {'p0': (Wacc_Protocol_P0,), 'p1': (Wacc_Protocol_P1,Wacc_Protocol_P0,)}
 
-    def startup(self):
+    def startup(self, threaded=False):
         """
         First determine which protocol version the uC firmware is running.
         Based on that version, replaces PimuBase class inheritance with a inheritance to a child class of PimuBase that supports that protocol
         """
-        WaccBase.startup(self)
+        WaccBase.startup(self, threaded=threaded)
         if self.hw_valid:
             if self.board_info['protocol_version'] in self.supported_protocols:
                 Wacc.__bases__ = self.supported_protocols[self.board_info['protocol_version']]
