@@ -14,6 +14,9 @@ class WristYaw(DynamixelHelloXL430):
                       'forward': deg_to_rad(0.0),
                       'stow': deg_to_rad(180.0)}
 
+    def startup(self, threaded=True):
+        return DynamixelHelloXL430.startup(self, threaded=threaded)
+
     def home(self):
         """
         Home to hardstops
@@ -35,6 +38,7 @@ class WristYaw(DynamixelHelloXL430):
         the commanded position from the current position, so as to lower the steady
         state error of the PID controller.
         """
+        DynamixelHelloXL430.step_sentry(self, robot)
         if self.hw_valid and self.robot_params['robot_sentry']['wrist_yaw_overload']:
             if self.status['stall_overload']:
                 if self.status['effort']>0:
